/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 *
 * @file fs_search.h
 * @brief Header file for text related definitions and classes.
 */

#ifndef FS_SEARCH_H_
#define FS_SEARCH_H_

#include "common/fs_common.h"
#include "pdf/annots/fs_annot.h"
#include "pdf/fs_pdfpage.h"
#include "addon/xfa/fs_xfa.h"

/**
 * @brief Foxit namespace.
 *
 * @details All the definitions and classes of Foxit PDF SDK are defined within this namespace.
 */
namespace foxit {
/**
 * @brief PDF namespace.
 */
namespace pdf {
/**
 * This class represents a callback object which can be used to cancel a searching process.
 * All the pure virtual functions in this class are used as callback functions and should be implemented by user.
 */
class SearchCancelCallback
{
 public:
  /**
   * @brief A callback function used to check whether to cancel the searching process or not.
   *
   * @return <b>true</b> means the searching process needs to be canceled, while <b>false</b> means
   *         the searching process does not need to be canceled.
   */
  virtual bool NeedToCancelNow() = 0;
};

/**
 * PDF text page represents all the text contents in a PDF page, according to a specified parsing flag for these text.
 * Class TextPage can be used to retrieve information about text in a PDF page, such as single character, single word,
 * text content within specified character range or rectangle and so on.<br>
 * This class object can also be used to construct objects of other text related classes in order to do more operation
 * for text contents or access specified information from text contents:
 * <ul>
 * <li>To search text in text contents of a PDF page, please construct a TextSearch object with TextPage object.</li>
 * <li>To access text that are used as a hypertext link, please construct a PageTextLinks object
 *     with TextPage object.</li>
 * </ul>
 *
 * @see TextSearch
 * @see PageTextLinks
 */
class TextPage FS_FINAL : public Base {
 public:
   /**
    * @brief Enumeration for parsing flags used for text page.
    *
    * @details Values of this enumeration can be used alone or in combination.
    */
   typedef enum _TextParseFlags {
     /** @brief No special parsing options for text page.*/
     e_ParseTextNormal = 0x0000,
     /** @brief Parse the text content of a PDF page with outputting the hyphen on a line feed.*/
     e_ParseTextOutputHyphen = 0x0001,
     /** @brief Parse the text content of a PDF page by the stream order.*/
     e_ParseTextUseStreamOrder = 0x0002
   } TextParseFlags;
   

  /**
   * @brief Constructor, from a parsed PDF page.
   *
   * @param[in] page   A valid PDF page object. This page should has been parsed.
   * @param[in] flags  Parsing flags for the text page. Please refer to values starting from
   *                   {@link TextPage::e_ParseTextNormal} and this can be one or combination of these values.
   */
  explicit TextPage(const PDFPage& page, int flags = foxit::pdf::TextPage::e_ParseTextNormal);

  /** @brief Destructor. */
  ~TextPage();
  /**
   * @brief Constructor, with another TextPage object.
   *
   * @param[in] other  Another TextPage object.
   */
  TextPage(const TextPage& other);
  /**
   * @brief Assign operator.
   *
   * @param[in] other  Another TextPage object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  TextPage& operator = (const TextPage& other);

  /**
   * @brief Equal operator.
   *
   * @param[in] other  Another TextPage object. This function will check if current object is equal to this one.
   *
   * @return <b>true</b> means equal, while <b>false</b> means not equal.
   */
  bool operator == (const TextPage& other) const;
  /**
   * @brief Not equal operator.
   *
   * @param[in] other  Another TextPage object. This function will check if current object is not equal to this one.
   *
   * @return <b>true</b> means not equal, while <b>false</b> means equal.
   */
  bool operator != (const TextPage& other) const;

  /**
   * @brief Check whether current object is empty or not.
   *
   * @details When the current object is empty, that means current object is useless.
   *
   * @return <b>true</b> means current object is empty, while <b>false</b> means not.
   */
  bool IsEmpty() const;

  /**
   * @brief Get the count of all the characters.
   *
   * @return Count of characters.
   */
  int GetCharCount() const;

  /**
   * @brief Get all the characters within a range specified by a start index and count.
   *
   * @param[in] start  Index of start character, which is the first character of the expected text content.
   *                   Valid range: from 0 to (<i>charcount</i> -1). <i>charcount</i> is returned by function
   *                   {@link TextPage::GetCharCount}. Default value: 0.
   * @param[in] count  Count of characters to be retrieved. -1 means to get the whole characters
   *                   from <i>start_index</i> to the end of PDF page. Especially, when parameter <i>count</i>
   *                   is larger than (<i>charcount</i> - start), all the rest character
   *                   (from <i>start_index</i>) will be retrieved. <i>charcount</i> is returned by function
   *                   {@link TextPage::GetCharCount}. Default value: -1.
   *
   * @return The characters within the specified character index range.
   */
  WString GetChars(int start = 0, int count = -1) const;

  /**
   * @brief Get the character index at or around a specified position on the page,
   *        in [PDF coordinate system] (@ref pdf::PDFPage).
   *
   * @param[in] x          Value of x position, in [PDF coordinate system] (@ref pdf::PDFPage).
   * @param[in] y          Value of y position, in [PDF coordinate system] (@ref pdf::PDFPage).
   * @param[in] tolerance  Tolerance value for character hit detection, in point units. This should not be a negative.
   *
   * @return Index of the character, which is at or nearby point (x,y), starting from 0. Specially, if there are
   *         several characters near by point (x, y), the smallest character index will be returned.
   *         If there is no character at or nearby the point, -1 will be returned.
   */
  int GetIndexAtPos(float x, float y, float tolerance) const;

  /**
   * @brief Get the text within a rectangle, in [PDF coordinate system] (@ref pdf::PDFPage).
   *
   * @param[in] rect  A rectangle region, in [PDF coordinate system] (@ref pdf::PDFPage).
   *
   * @return Text string within the specified rectangle.
   */
  WString GetTextInRect(const RectF& rect) const;

  /**
   * @brief Get the character range of a word at or around a specified position on the page,
   *        in [PDF coordinate system] (@ref pdf::PDFPage).
   *
   * @details Currently, for Chinese/Japanese/Korean, only support to get a single character at or
   *          around the specified position.
   *
   * @param[in] x          Value of x position, in [PDF coordinate system] (@ref pdf::PDFPage).
   * @param[in] y          Value of y position, in [PDF coordinate system] (@ref pdf::PDFPage).
   * @param[in] tolerance  Tolerance value for word hit detection, in point units.This should not be a negative.
   *
   * @return The character range that represents the expected word. There would be at most one valid range segment
   *         in this Range object. If returned Range object is empty, that means no such word is found.
   */
  common::Range GetWordAtPos(float x, float y, float tolerance) const;

  /**
   * @brief Count the text rectangles within a range specified by a start index and count.
   *
   * @param[in] start  Index of start character in the character index range.
   *                   Valid range: from 0 to (<i>charcount</i> -1). <i>charcount</i> is returned by function
   *                   {@link TextPage::GetCharCount}.
   * @param[in] count  Count of characters in the character index range. -1 means to get the whole characters
   *                   from <i>start_index</i> to the end of PDF page.
   *
   * @return The count of text rectangles in the specified character index range. -1 means error.
   */
  int GetTextRectCount(int start = 0, int count = -1);

  /**
   * @brief Get the text rectangle by the index.
   *
   * @param[in] rect_index  The index of the rectangle to be retrieved.
   *                        Valid range: from 0 to (<i>count</i> -1). <i>count</i> is returned by function
   *                        {@link TextPage::GetTextRectCount}.
   *
   * @return A specified text rectangle.
   */
  RectF GetTextRect(int rect_index) const;

  /**
   * @brief Get the text trend (as rotation) of a specified rectangle.
   *
   * @param[in] rect_index  The index of the rectangle to be retrieved.
   *                        Valid range: from 0 to (<i>count</i> -1). <i>count</i> is returned by function
   *                        {@link TextPage::GetTextRectCount}.
   *
   * @return Text trend, as rotation value. Please refer to values starting from {@link common::e_Rotation0} and
   *         this would be one of these values.
   */
  common::Rotation GetBaselineRotation(int rect_index);

  /**
   * @brief Get the array of all text rectangles within the specified rectangle region.
   *
   * @param[in] rect  A rectangle region, in [PDF coordinate system] (@ref pdf::PDFPage).
   *
   * @return Text rectangle array within the specified rectangle.
   */
  RectFArray GetTextRectArrayByRect(const RectF rect);

  // User is strongly recommended NOT to use this method; otherwise unknown situation may occur.
  explicit TextPage(FS_HANDLE handle =  NULL);
};

/**
 * This class can construct text search in a PDF document/an XFA document/a text page or
 * in a PDF annotation's appearance. It offers functions to do a text search and get the searching result:
 * <ul>
 * <li>To specify the searching pattern and options, use functions {@link TextSearch::SetPattern},
 *     {@link TextSearch::SetStartPage} (only useful for a text search in PDF document),
 *     {@link TextSearch::SetEndPage} (only useful for a text search in PDF document) and
 *     {@link TextSearch::SetSearchFlags}.</li>
 * <li>To do the searching, use function {@link TextSearch::FindNext} or {@link TextSearch::FindPrev}.</li>
 * <li>To get the searching result, use functions GetMatchXXX().</li>
 * </ul>
 */
class TextSearch FS_FINAL : public Base {
 public:
  /**
   * @brief Enumeration for searching flags.
   *
   * @details Values of this enumeration can be used alone or in combination.
   */
  typedef enum _SearchFlags {
    /** @brief No special searching options.*/
    e_SearchNormal        = 0x00,
    /** @brief If set, match the case of keyword when searching. */
    e_SearchMatchCase      = 0x01,
    /** @brief If set, match the whole word of keyword when searching. */
    e_SearchMatchWholeWord    = 0x02,
    /**
     * @brief If set, match the key word consecutively when searching. For example, "CC" will be matched twice in "CCC".
     */
    e_SearchConsecutive     = 0x04
  } SearchFlags;
  

  /**
   * @brief Constructor, for a PDF document.
   *
   * @details User can set a valid SearchCancelCallback object in order to decide whether to cancel the searching
   *          process or not when the callback function in SearchCancelCallback is triggered.
   *
   * @param[in] document  A valid PDF document object.
   * @param[in] cancel    A SearchCancelCallback object which decides if the searching process needs to be canceled
   *                      when the callback function is triggered. This can be <b>NULL</b> which means not to
   *                      cancel the searching process. If this is not <b>NULL</b>, it should be a valid
   *                      SearchCancelCallback object implemented by user. Default value: <b>NULL</b>.
   */
  explicit TextSearch(const PDFDoc& document, SearchCancelCallback* cancel = NULL);

  /**
   * @brief Constructor, for a specified XFA document.
   *
   * @param[in] xfa_document  A valid XFA document object. It should have been loaded by
   *                          function {@link addon::xfa::XFADoc::StartLoad}.
   * @param[in] cancel        A SearchCancelCallback object which decides if the searching process needs to be canceled
   *                          when the callback function is triggered. This can be <b>NULL</b> which means not to
   *                          cancel the searching process. If this is not <b>NULL</b>, it should be a valid
   *                          SearchCancelCallback object implemented by user. Default value: <b>NULL</b>.
   */
  explicit TextSearch(const foxit::addon::xfa::XFADoc& xfa_document, foxit::pdf::SearchCancelCallback* cancel = NULL);

  /**
   * @brief Constructor, for a specified text page.
   *
   * @param[in] text_page  A text page object.
   */
  explicit TextSearch(const foxit::pdf::TextPage& text_page);

  /**
   * @brief Constructor, for a specified PDF annotation.
   *
   * @details Currently, only support to search in appearance of following annotation types:
   *          free text annotation, stamp annotation, widget annotation, stamp annotation, and line annotation.
   *
   * @param[in] annot  A valid annotation object.
   */
  explicit TextSearch(const foxit::pdf::annots::Annot& annot);

  /** @brief Destructor. */
  ~TextSearch();
  /**
   * @brief Constructor, with another TextSearch object.
   *
   * @param[in] other  Another TextSearch object.
   */
  TextSearch(const TextSearch& other);
  /**
   * @brief Assign operator.
   *
   * @param[in] other  Another TextSearch object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  TextSearch& operator = (const TextSearch& other);

  /**
   * @brief Equal operator.
   *
   * @param[in] other  Another TextSearch object. This function will check if current object is equal to this one.
   *
   * @return <b>true</b> means equal, while <b>false</b> means not equal.
   */
  bool operator == (const TextSearch& other) const;
  /**
   * @brief Not equal operator.
   *
   * @param[in] other  Another TextSearch object. This function will check if current object is not equal to this one.
   *
   * @return <b>true</b> means not equal, while <b>false</b> means equal.
   */
  bool operator != (const TextSearch& other) const;

  /**
   * @brief Check whether current object is empty or not.
   *
   * @details When the current object is empty, that means current object is useless.
   *
   * @return <b>true</b> means current object is empty, while <b>false</b> means not.
   */
  bool IsEmpty() const;

  /**
   * @brief Set keywords to search.
   *
   * @param[in] key_words  The text content to be searched. It should not be an empty string.
   *
   * @return <b>true</b> means success, while <b>false</b> means failure.
   */
  bool SetPattern(const wchar_t* key_words);

  /**
   * @brief Set starting page index.
   *
   * @details This function can only be used for a text search in PDF document or in XFA document.
   *          For other case, this function will throw exception {@link foxit::e_ErrUnsupported}.<br>
   *          If this function is not called, default value 0 will be used as the starting page index.
   *
   * @param[in] page_index  Index of the page, from which the search starts.
   *                        Valid range: from 0 to (<i>count</i>-1). <i>count</i> is returned by function
   *                        {@link pdf::PDFDoc::GetPageCount} for PDF document or by function
   *                        {@link addon::xfa::XFADoc::GetPageCount} for XFA document.
   *
   * @return <b>true</b> means success, while <b>false</b> means failure.
   */
  bool SetStartPage(int page_index);

  /**
   * @brief Set ending page index.
   *
   * @details This function can only be used for a text search in PDF document or in XFA document.
   *          For other case, this function will throw exception {@link foxit::e_ErrUnsupported}.<br>
   *          If this function is not called, default value <i>count</i>-1 will be used as the ending page index.
   *
   * @param[in] page_index  Index of the page, from which the search ended.
   *                        Valid range: from 0 to (<i>count</i>-1). <i>count</i> is returned by function
   *                        {@link pdf::PDFDoc::GetPageCount} for PDF document or by function
   *                        {@link addon::xfa::XFADoc::GetPageCount} for XFA document.
   *
   * @return <b>true</b> means success, while <b>false</b> means failure.
   */
  bool SetEndPage(int page_index);

  /**
   * @brief Set search flags.
   *
   * @details If this function is not called, default value {@link TextSearch::e_SearchNormal}
   *          will be used.
   *
   * @param[in] search_flags  Search flags. Please refer to values starting from {@link TextSearch::e_SearchNormal}
   *                          and this can be one or combination of these values.
   *
   * @return <b>true</b> means success, while <b>false</b> means failure.
   */
  bool SetSearchFlags(uint32 search_flags);

  /**
   * @brief Search for next matched pattern.
   *
   * @return <b>true</b> means the next match is found, while <b>false</b> means no next match can be found or
   *         any other error.
   */
  bool FindNext();

  /**
   * @brief Search for previous matched pattern.
   *
   * @return <b>true</b> means the previous match is found, while <b>false</b> means no previous match can be found or
   *         any other error.
   */
  bool FindPrev();

  /**
   * @brief Get the rectangles of current match pattern.
   *
   * @return An array of rectangles.
   */
  RectFArray GetMatchRects() const;

  /**
   * @brief Get the page index, to which current match belongs.
   *
   * @details This function can only be used for a text search in PDF document or in XFA document.
   *
   * @return The index of the page, which contains the current match, starting from 0.
   *         -1 means not matched pattern has been found.
   */
  int GetMatchPageIndex() const;

  /**
   * @brief Get the sentence that contains current match pattern.
   *
   * @return The sentence content, which contains current match.
   */
  WString GetMatchSentence();

  /**
   * @brief Get the index of the first character of current match pattern, based on the match sentence.
   *
   * @details In a sentence, there may be more than 2 match patterns. This function can help to confirm which pattern
   *          in the sentence is just current match pattern.
   *
   * @return The index of the first character of current match pattern, based on the match sentence, starting from 0.
   *         -1 means no matched sentence is found.
   */
  int GetMatchSentenceStartIndex();

  /**
   * @brief Get the index of the first character of current match pattern, based on current match page.
   *
   * @return The index of the first character, in current match page, starting from 0.
   *         -1 means no matched sentence is found.
   */
  int GetMatchStartCharIndex() const;

  /**
   * @brief Get the index of the last character of current match pattern, based on current match page.
   *
   * @return The index of the last character, in current match page, starting from 0.
   *         -1 means no matched sentence is found.
   */
  int GetMatchEndCharIndex() const;

  // User is strongly recommended NOT to use this method; otherwise unknown situation may occur.
  explicit TextSearch(FS_HANDLE handle =  NULL);
};

/**
 * A text link is just text content, which represents a hypertext link to a website or a resource on the internet,
 * or an e-mail address.<br>
 * This class offers functions to get information from the text link.<br>
 * A TextLink object is retrieved by function {@link PageTextLinks::GetTextLink}.
 *
 * @see PageTextLinks
*/
class TextLink FS_FINAL : public Base{
 public:
  /** @brief Destructor. */
  ~TextLink();
  /**
   * @brief Constructor, with another TextLink object.
   *
   * @param[in] other  Another TextLink object.
   */
  TextLink(const TextLink& other);
  /**
   * @brief Assign operator.
   *
   * @param[in] other  Another TextLink object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  TextLink& operator = (const TextLink& other);

  /**
   * @brief Equal operator.
   *
   * @param[in] other  Another TextLink object. This function will check if current object is equal to this one.
   *
   * @return <b>true</b> means equal, while <b>false</b> means not equal.
   */
  bool operator == (const TextLink& other) const;
  /**
   * @brief Not equal operator.
   *
   * @param[in] other  Another TextLink object. This function will check if current object is not equal to this one.
   *
   * @return <b>true</b> means not equal, while <b>false</b> means equal.
   */
  bool operator != (const TextLink& other) const;

  /**
   * @brief Check whether current object is empty or not.
   *
   * @details When the current object is empty, that means current object is useless.
   *
   * @return <b>true</b> means current object is empty, while <b>false</b> means not.
   */
  bool IsEmpty() const;

  /**
   * @brief Get URI string.
   *
   * @details A text link's URI can be a hypertext link to a website or a resource on the internet.
   *          It can also be an e-mail address.
   *
   * @return URI string.
   */
  WString GetURI();

  /**
   * @brief Get the index of start character of current hyper-link URI, based on PDF page.
   *
   * @return The index of start character, starting from 0.
   */
  int GetStartCharIndex();

  /**
   * @brief Get the index of last character of current hyper-link URI, based on PDF page.
   *
   * @return The index of last character, starting from 0.
   */
  int GetEndCharIndex();

  /**
   * @brief Get the text rectangles of current text link object.
   *
   * @return An array of rectangles.
   */
  RectFArray GetRects();
  // User is strongly recommended NOT to use this method; otherwise unknown situation may occur.
  explicit TextLink(FS_HANDLE handle =  NULL);

};

/**
 * In a PDF page, some text content may represent a hypertext link to a website or a resource on the internet,
 * or an e-mail address. Class PageTextLinks is used to manage these kind of text content as text links.
 */
class PageTextLinks FS_FINAL : public Base{
 public:
  /**
   * @brief Constructor, from a parsed PDF page.
   *
   * @param[in] page  A valid PDF page object. This page should has been parsed.
   */
  explicit PageTextLinks(const TextPage& page);
  /**
   * @brief Constructor, with another PageTextLinks object.
   *
   * @param[in] other  Another PageTextLinks object.
   */
  PageTextLinks(const PageTextLinks& other);
  /**
   * @brief Assign operator.
   *
   * @param[in] other  Another PageTextLinks object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  PageTextLinks& operator = (const PageTextLinks& other);
  /**
   * @brief Equal operator.
   *
   * @param[in] other  Another PageTextLinks object. This function will check if current object is equal to this one.
   *
   * @return <b>true</b> means equal, while <b>false</b> means not equal.
   */
  bool operator == (const PageTextLinks& other) const ;
  /**
   * @brief Not equal operator.
   *
   * @param[in] other  Another PageTextLinks object. This function will check if current object is not equal to this one.
   *
   * @return <b>true</b> means not equal, while <b>false</b> means equal.
   */
  bool operator != (const PageTextLinks& other) const ;

  /**
   * @brief Check whether current object is empty or not.
   *
   * @details When the current object is empty, that means current object is useless.
   *
   * @return <b>true</b> means current object is empty, while <b>false</b> means not.
   */
  bool IsEmpty() const;
  /** @brief Destructor. */
  ~PageTextLinks();

  /**
   * @brief Get the count of the URL formatted texts, in related PDF page.
   *
   * @return The count of the text links, if returned -1 means error on the construct.
   */
  int GetTextLinkCount();

  /**
   * @brief Get a text link object by index.
   *
   * @param[in] index  Index of a text link object. Valid range: from 0 to (<i>count</i>-1).
   *                   <i>count</i> is returned by function {@link PageTextLinks::GetTextLinkCount}.
   *
   * @return The specified text link object.
   */
  TextLink GetTextLink(int index);

  // User is strongly recommended NOT to use this method; otherwise unknown situation may occur.
  explicit PageTextLinks(FS_HANDLE handle =  NULL);
};
}  // namespace pdf
}  // namespace foxit
#endif  // FS_SEARCH_H_

